@@ -13,4 +13,4 @@ class Config:
             setattr(self, key, value)
     
     def __getitem__(self, key):
-        return self.settings[key]
\ No newline at end of file
+        return self.settings.get(key, None)
\ No newline at end of file
