@@ -62,7 +62,7 @@ class Files(Base, CRUD):
 
         PATH = os.path.join(PATH, str(uuid.uuid4()))    
 
-        self.url = os.path.join(Config()["pyramid_rest.files.upload_url"] or "/upload/",PATH, name)
+        self.url = os.path.join(Config()["pyramid_rest.files.static_url"] or "/upload/", PATH, name)
 
         PATH = os.path.join(Config()["pyramid_rest.files.upload_path"], 
             PATH)
