@@ -20,10 +20,16 @@ class Logger:
                        'files',
                        'models',
                        'events',
-                       'request_manager']
+                       'request_manager',
+                       'admin', 
+                       'security', 
+                       'querying']
     def __init__(self, app = None):
         for name in self.ALLOWED_LOGGERS:
-            setattr(self, name, logging.getLogger(name))
+            setattr(self, name, logging.getLogger("pyramid_rest").getChild(name))
+            
+    def parse_config(self, fileName):
+        logging.config.fileConfig(fileName)
             
             
         
