@@ -1,6 +1,11 @@
 #-*- coding: utf-8 -*-
 import sqlahelper
-from sqlalchemy import types
+from sqlalchemy import types, event
+from sqlalchemy.schema import Column
+from pprint import pprint, pformat
+from sqlalchemy.types import DateTime, Integer
+import time
+import datetime
 FlAG_PRIMARY_KEY = "primary key"
 FLAG_UNIQUE = "unique"
 
@@ -89,7 +94,40 @@ def json(self, request):
             out[key] = value
     return out
 
+def _listen_on_insert(mapper, connection, target):
+    target.created = datetime.datetime.now()
+    target.updated = datetime.datetime.now()
+    
+def _listen_on_update(mapper, connection, target):
+    target.updated = datetime.datetime.now()
+    
+
+class CUTimes(object):
+    u"""
+        При наследовании от данного класса, автоматически создаются поля:
+        
+            * created - datetime создания объекта
+            * updated - datetime изменения объекта
+        
+        Автоматически навешиваются события на before_insert и before_update
+        
+    """
+    _EVENTS_IS_BINDED = False
+    
+    created = Column(DateTime)
+    updated = Column(DateTime)
+    
+    def __new__(cls, *args, **kwargs):
+        if not cls._EVENTS_IS_BINDED:
+            
+            event.listen(cls, "before_insert", _listen_on_insert)
+            event.listen(cls, "before_update", _listen_on_update)
+            
+            cls._EVENTS_IS_BINDED = True
+        return super(CUTimes, cls).__new__(cls, *args, **kwargs)
+
 class CRUD:
+    
     def _add(self):
         sqlahelper.get_session().add(self)
         return self
@@ -170,6 +208,22 @@ class CRUD:
                     value = unicode(value)
                 out[key] = value
         return out
+    
+    @classmethod
+    def columns(cls):
+        out = []
+        for key, value in cls.__dict__.iteritems():
+            if key[0] != "_":
+                out += [key]
+        return out
+    
+    @classmethod
+    def get_schema(cls):
+        for column, info in cls.__table__._columns._data.iteritems():
+            print column, pformat(info.__dict__)
+            print "\n\n"
+
+
 
 def constructSuperModel(clas):
     """Constructs super model"""
