@@ -14,6 +14,9 @@ import json
 from pyramid_rest.config import Config
 from pyramid_rest.events import EventManager
 from pyramid_rest.logger import Logger
+from pyramid_rest.exceptions import UnknownError
+from pyramid.view import view_config
+
 
 
 class HandlerInterface:
@@ -65,7 +68,7 @@ class email(HandlerInterface):
         self._parse_config()
         request = request.request
         BODY = u"""
-            Критическая ошибка на %(full_path)s.
+            Code: %(code)s на %(full_path)s.
             <br>
             <h3>Session info:</h3>
             <ul>
@@ -97,7 +100,8 @@ class email(HandlerInterface):
             
             
                 
-            """%({"full_path": request["PATH_INFO"],
+            """%({"code": code,
+                  "full_path": request["PATH_INFO"],
                   "ip": get_ip(request),
                   "dt": datetime.datetime.now(),
                   "email": get_or_none(request, "user", "email"),
@@ -108,12 +112,63 @@ class email(HandlerInterface):
                   "headers": pformat(dict(request.headers)),
                   "trace": traceback.format_exc()})
         send_message(request, 
-                     subject = u"Критическая ошибка на %s %s."%(request.headers["host"], 
-                                                                request["PATH_INFO"]),
+                     subject = u"%s на %s %s."%(code,
+                                                request.headers["host"], 
+                                                request["PATH_INFO"]),
                      sender = self.sender,
                      recipients = self.recipients,
                      html = BODY)
 
+class console(HandlerInterface):
+    def handle_response(self, request, code, body):
+        wrapped_request = request
+        request = request.request
+        Logger().request_manager.info("""Handled response in console with metainfo: 
+        code: %(code)s
+        full_path: %(full_path)s
+        
+        wrapped request data:
+        
+            %(wrdata)s
+        
+        Session info:
+            Datetime: %(dt)s
+            Full path:  %(full_path)s
+            User IP: %(ip)s
+            User email: %(email)s
+        
+        Trace:
+            %(trace)s
+            
+        Request type:
+            %(full_path)s
+            %(request_type)s
+        
+        Get data:
+            %(get)s
+        
+        Post data:
+            %(post)s
+            
+        Headers:
+            %(headers)s
+
+        Request:
+            %(request)s
+            
+        """%({"code": code,
+                  "full_path": request["PATH_INFO"],
+                  "ip": get_ip(request),
+                  "dt": datetime.datetime.now(),
+                  "email": get_or_none(request, "user", "email"),
+                  "request": pformat(request.__dict__),
+                  "request_type": request.method,
+                  "get": pformat(dict(request.GET)),
+                  "post": pformat(dict(request.POST)),
+                  "headers": pformat(dict(request.headers)),
+                  "trace": traceback.format_exc(), 
+                  "wrdata": pformat(wrapped_request.data)}))
+
 @singletone
 class ExceptionManager:
     """
@@ -140,7 +195,8 @@ class ExceptionManager:
     """
     
     HANDLER_MAPPER = {"db": db, 
-                      "email": email}
+                      "email": email, 
+                      "console": console}
     
     def __init__(self):
         pass
@@ -168,22 +224,26 @@ class ExceptionManager:
             Именно эта функция прикручивается к обработчику события **before-response**
         """
         code = request.request.response.status_code
+        response = response.data
         
         if not self.use:
             return 
-        if code not in self.watch_codes:
+        if self.watch_codes != [] and code not in self.watch_codes:
             return
         if code in self.unwatch_codes:
             return
         
         handlers = self.handlers.get(str(code), None)
+        if handlers is None:
+            handlers = self.handlers.get("*", None)
         
-        for handler in handlers:
-            if handler is None:
-                Logger().request_manager.error("Code watched, but no handlers found for it. Locals: \n")
-                return 
-            
-            self._get_handler(handler)().handle_response(request, code, response)
+        if handlers:
+            for handler in handlers:
+                if handler is None:
+                    Logger().request_manager.error("Code watched, but no handlers found for it. Locals: \n")
+                    return 
+                
+                self._get_handler(handler)().handle_response(request, code, response)
         
         
         
