@@ -9,26 +9,34 @@ class Resource:
             @routeRegister("/users")        #привязка класса к роуту
             class UserView(Resource):
             
-                def get(self):
+                def get(self, request):
                     "Реализует get интерфейс для /users"
-                def post(self):
+                def post(self, request):
                     "Реализует post интерфейс для /users"
                 
-                def delete(self):
+                def delete(self, request):
                     "реализует delete для /users"
                 
-                def get_object(self):
+                def get_object(self, request):
                     "реализует get для /users/{id}"
                 
-                def put_obj(self):
+                def put_object(self, request):
                     "реализует post для /users/{id}"
                 
-                def delete_obj(self):
+                def delete_object(self, request):
                     "реализует delete для /users/{id}
         
         .. warning::
         
             Метод PUT для табличного и POST для объектного роутов **запрещены**!
+            
+        .. note::
+        
+            * Каждый метод принимает объект :class:`pyramid_rest.request.Request`
+                
+            * Методы с суффиксом object обрабатывают объектные роуты
+            
+        :param app: текущий экземпляр :class:`pyramid_rest.app.App`
         
     """
     def __init__(self, app):
@@ -37,20 +45,12 @@ class Resource:
     
     def get_table_view(self):
         """
-            Возвращает метод для обработки табличных роутов.
-            Этот метод из обычного pyramid request создает экземпляр класса
-            :class:`python_rest.request.Request`. Этот экземпляр созхраняется как self.request. Оригинальный pyramid request 
-            сохраняется в self.pyramid_request. Далее пробуем из методов класса выбрать метод, который совпадает с названием
-            HTTP метода запроса. Если не получается, генерим :class:`pyramid_rest.exceptions.MethodNotAllowed`, иначе возвращаем
-            массив объектов
+            Возвращает callback для обработки текущего запроса согласно методу запроса для табличного роута.
         """
         def _table_view(request):
-            self.request = request
-            self.pyramid_request = request.request
-            
             self.app.logger.resources.info("Called resource %s", self)
             
-            method = self.request.get_method()
+            method = request.get_method()
             
             if method not in ("get", "post", "delete"):
                 raise MethodNotAllowed(info = "Method %s not allowed"%(method))
@@ -58,25 +58,18 @@ class Resource:
             if not hasattr(self, method):
                 raise MethodNotAllowed(info = "Method %s not allowed"%(method))
             
-            return getattr(self, method)()
+            return getattr(self, method)(request)
         return _table_view
     
     def get_object_view(self):
         """
-            Возвращает метод для обработки объектных роутов.
-            Этот метод из обычного pyramid request создает экземпляр класса
-            :class:`python_rest.request.Request`. Этот экземпляр созхраняется как self.request. Оригинальный pyramid request 
-            сохраняется в self.pyramid_request. Далее пробуем из методов класса выбрать метод, который совпадает с названием
-            HTTP метода запроса. Если не получается, генерим :class:`pyramid_rest.exceptions.MethodNotAllowed`, иначе возвращаем
-            массив объектов
+            Возвращает callback для обработки текущего запроса согласно методу запроса для табличного роута.
         """
         def _table_view(request):
-            self.request = request
-            self.pyramid_request = request.request
             
             self.app.logger.resources.info("Called resource %s", self)
             
-            method = self.request.get_method()
+            method = request.get_method()
             
             if method not in ("get", "put", "delete"):
                 raise MethodNotAllowed(info = "Method %s not allowed"%(method))
@@ -86,7 +79,7 @@ class Resource:
             if not hasattr(self, view_name):
                 raise MethodNotAllowed(info = "Method %s not allowed"%(method))
             
-            return getattr(self, view_name)()
+            return getattr(self, view_name)(request)
         return _table_view
     
     
