@@ -1,8 +1,7 @@
 #-*- coding: utf-8 -*-
-from pyramid_rest.exceptions import Forbidden
+from pyramid_rest.exceptions import Forbidden, Unauthorized
 import transaction
 from pyramid_rest.security.models import Sessions
-from pyramid_rest.autotests import AutoTestManager
 from pprint import pformat, pprint
 from pyramid_rest.helpers import get_class_that_defined_method
 
@@ -21,46 +20,30 @@ class AllowToGroups:
         self.cls = kwargs.get("cls", None)
     
     def __call__(self, resource_view):
-        def _callback(self_resource):
-            request = getattr(self_resource, "request", None) or self_resource
+        def _callback(self_resource, request):
             with transaction.manager:
                 SID = Sessions._get(sid = request.cookies.get("SID", None))
                 if SID is None or SID.user is None:
-                    raise Forbidden(info = {"allowed_groups": self.groups,
+                    raise Unauthorized(info = {"allowed_groups": self.groups,
                                             "your_group": None})
                 else:
                     db_user = SID.user
                     if not db_user:
-                        raise Forbidden(info = {"allowed_groups": self.groups,
+                        raise Unauthorized(info = {"allowed_groups": self.groups,
                                                 "your_group": None})
                     
                     request.set_attr("userid", db_user.id)
                     request.set_attr("sid", SID.sid)
                     
                     if self.groups[0] == "all":
-                        return resource_view(self_resource)
+                        return resource_view(self_resource, request)
                     
                     if db_user.group not in self.groups:
                         raise Forbidden(info = {"allowed_groups": self.groups,
                                                 "your_group": db_user.group})
                     
                     
-                return resource_view(self_resource)
-        
-        if AutoTestManager().search(view_name = resource_view.__name__, 
-                                             resource = self.cls) == []:
-            AutoTestManager().META += [{"resource": self.cls,
-                                        "method": resource_view.__name__,
-                                        "url": None,
-                                        "view": _callback,
-                                        "auth_required": True,
-                                        "auth_groups":self.groups,
-                                        "view_name": resource_view.__name__}]
-        
-        for item in AutoTestManager().search(view_name = resource_view.__name__, 
-                                             resource = self.cls):
-            item["auth_required"] = True
-            item["auth_groups"] = self.groups
+                return resource_view(self_resource, request)
         
         _callback.__doc__ = resource_view.__doc__ or ""
         _callback.__name__ = resource_view.__name__
