@@ -2,6 +2,11 @@
 from pyramid_rest.helpers import singletone
 from pyramid_rest.view import View
 from pyramid_rest.events import EventManager
+import json
+from pyramid.renderers import JSON
+
+def _utf_8_json_renderer(obj, **kw):
+    return json.dumps(obj, ensure_ascii=False, **kw)
 
 class Server:
     def __init__(self, app):
