@@ -2,6 +2,7 @@
 from pyramid_rest.exceptions import ExceptionInterface
 from pyramid_rest.events import EventManager
 from pyramid_rest.request import Request
+from pyramid_rest.response import Response
 
 
 class View:
@@ -22,11 +23,16 @@ class View:
         
         
     def __call__(self, pyramid_request):
-        request = Request(pyramid_request)
-        EventManager().trigger("before-request", self, request)
         try:
+            request = Request(pyramid_request)
+            EventManager().trigger("before-request", self, request)
             response = self.view(request)
+            code = 200
         except ExceptionInterface as x:
             response = x.serialize(request)
-        EventManager().trigger("before-response", self, request, response)
-        return response
\ No newline at end of file
+            code = x.code
+            
+        responseClass = Response(request, response, code)
+        EventManager().trigger("before-response", self, request, responseClass)
+        
+        return responseClass.serialize()
\ No newline at end of file
